//CW3DMilkshapeModel.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

#include "../headers/CW3DMilkshapeModel.h"

//constructor and deconstructor
//sets default values and releases memory
CW3DMilkshapeModel::CW3DMilkshapeModel()
{
	//set defaults
	m_pGroups=0;
	m_NumGroups=0;
}
CW3DMilkshapeModel::~CW3DMilkshapeModel()
{
	//release memory
	Destroy();
}
//this will load the model into the model group structures
//and set everything up to render the static mesh
HRESULT CW3DMilkshapeModel::Initialize(char* strFilename,char* strRepathLocation,IDirect3DDevice8* pDevice)
{
	//destroy the class if it was already initialized
	Destroy();

	//first, convert the filename of the file using repath location
	char* strFile=RepathFilename(strFilename,strRepathLocation);

	//next load the milkshape model into a milkshape data structure
	sMilkshapeData* pMilkshapeData=LoadMilkshapeModel(strFile);

	//make sure it loaded correctly with no errors
	if(pMilkshapeData==0)
		return(E_FAIL);

	//now create the model groups and copy the number
	//of model groups into the member variable
	m_NumGroups=pMilkshapeData->NumGroups;
	m_pGroups=new sModelGroup[pMilkshapeData->NumGroups];

	//loop through the model groups and load the model group data
	for(int i=0;i<pMilkshapeData->NumGroups;i++)
	{
		//set the material in the group
		int MaterialIndex=pMilkshapeData->pGroups[i].MaterialIndex;
		m_pGroups[i].Material=pMilkshapeData->pMaterials[MaterialIndex].Material;

		//repath the texture filename and load up the texture
		char* Filename=RepathFilename(pMilkshapeData->pMaterials[MaterialIndex].TextureFilename,strRepathLocation);
		if(FAILED(D3DXCreateTextureFromFile(pDevice,Filename,&m_pGroups[i].Texture)))
			m_pGroups[i].Texture=0;

		//set the number of vertices
		m_pGroups[i].NumFaces=pMilkshapeData->pGroups[i].NumFaces;

		//loop through and load the vertices
		MS3DMODELVERTEX* pVertices=new MS3DMODELVERTEX[m_pGroups[i].NumFaces*3];
		for(int j=0;j<m_pGroups[i].NumFaces*3;j++)
		{
			//get the vertex and triangle pointer
			sFace* pFace=&pMilkshapeData->pFaces[pMilkshapeData->pGroups[i].pFaceIndices[j/3]];
			sVertex* pVert=&pMilkshapeData->pVertices[pFace->VertexIndices[j%3]];

			//copy the data
			pVertices[j].pos=pVert->vecPos;
			pVertices[j].nrm=pFace->VertexNormals[j%3];
			pVertices[j].tu=pFace->tu[j%3];
			pVertices[j].tv=pFace->tv[j%3];
		}

		//set the material and create the vertex buffer
		//setting the material when we create the vertex buffer
		//will defaultly set a material for the vertices. in other
		//words, we don't need to store a diffuse and specular values
		//in the vertex because they're stored in the vertex buffer
		if(FAILED(hr=pDevice->SetMaterial(&m_pGroups[i].Material)))
			return(hr);
		if(FAILED(hr=pDevice->CreateVertexBuffer(sizeof(MS3DMODELVERTEX)*m_pGroups[i].NumFaces*3,//size
									D3DUSAGE_WRITEONLY,//usage
									D3DFVF_MS3DMODELVERTEX,//fvf
									D3DPOOL_MANAGED,//pool
									&m_pGroups[i].VertexBuffer)))
		{
			m_pGroups[i].VertexBuffer=0;
			return(hr);
		}

		//lock the buffer and copy the vertices, then unlock
		void* Data;
		if(FAILED(hr=m_pGroups[i].VertexBuffer->Lock(0,0,(BYTE**)&Data,0)))
			return(hr);
		memcpy(Data,pVertices,sizeof(MS3DMODELVERTEX)*m_pGroups[i].NumFaces*3);
		if(FAILED(hr=m_pGroups[i].VertexBuffer->Unlock()))
			return(hr);

		//compute the bounding sphere and bounding box
		D3DXVECTOR3 vec1,vec2;
		D3DXComputeBoundingSphere(pVertices,m_pGroups[i].NumFaces*3,
			D3DFVF_MS3DMODELVERTEX,&vec1,&vec2.x);
		m_pGroups[i].BoundingSphere.x=vec1.x;
		m_pGroups[i].BoundingSphere.y=vec1.y;
		m_pGroups[i].BoundingSphere.z=vec1.z;
		m_pGroups[i].BoundingSphere.r=vec2.x;
		D3DXComputeBoundingBox(pVertices,m_pGroups[i].NumFaces*3,
			D3DFVF_MS3DMODELVERTEX,&vec1,&vec2);
		m_pGroups[i].BoundingBox.x=(vec1.x+vec2.x)/2;
		m_pGroups[i].BoundingBox.y=(vec1.y+vec2.y)/2;
		m_pGroups[i].BoundingBox.z=(vec1.z+vec2.z)/2;
		m_pGroups[i].BoundingBox.w=vec2.x-vec1.x;
		m_pGroups[i].BoundingBox.h=vec2.y-vec1.y;
		m_pGroups[i].BoundingBox.d=vec2.z-vec1.z;

		//delete teh vertices now that we don't need them anymore
		if(pVertices)
			delete[] pVertices;
	}

	//delete the data structure
	delete pMilkshapeData;

	//return success
	return(S_OK);
}
//this renders the model. it loops through all the model groups, set the texture
//and material, checks if it's within the frustrum if any, and renders the buffer
HRESULT CW3DMilkshapeModel::Render(CW3DFrustum* pFrustum,bool bCheckBoundingBox,bool bCheckBoundingSphere,IDirect3DDevice8* pDevice)
{
	//loop through each group and render
	for(int i=0;i<m_NumGroups;i++)
	{
		bool Render=true;
		//check if they're within the viewing frustrum
		if(bCheckBoundingBox)
			if(pFrustum->IsBoxInFrustum(	m_pGroups[i].BoundingBox.x,
											m_pGroups[i].BoundingBox.y,
											m_pGroups[i].BoundingBox.z,
											m_pGroups[i].BoundingBox.w,
											m_pGroups[i].BoundingBox.h,
											m_pGroups[i].BoundingBox.d)==false)
				Render=false;
		if(bCheckBoundingSphere)
			if(pFrustum->IsSphereInFrustum(	m_pGroups[i].BoundingSphere.x,
												m_pGroups[i].BoundingSphere.y,
												m_pGroups[i].BoundingSphere.z,
												m_pGroups[i].BoundingSphere.r)==false)
				Render=false;

		//check if it should render
		if(Render)
		{
			//set the texture if any
			if(m_pGroups[i].Texture)
			{
				if(FAILED(hr=pDevice->SetTexture(0,m_pGroups[i].Texture)))
					return(hr);
			}
			else
			{
				if(FAILED(hr=pDevice->SetTexture(0,0)))
					return(hr);
			}

			//set the material
			if(FAILED(hr=pDevice->SetMaterial(&m_pGroups[i].Material)))
				return(hr);

			//render the vertex buffer
			if(FAILED(hr=pDevice->SetStreamSource(0,m_pGroups[i].VertexBuffer,sizeof(MS3DMODELVERTEX))))
				return(hr);
			if(FAILED(hr=pDevice->SetVertexShader(D3DFVF_MS3DMODELVERTEX)))
				return(hr);
			if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,m_pGroups[i].NumFaces)))
				return(hr);
		}
	}
	//return success
	return(S_OK);
}
//releases the memory for the class
HRESULT CW3DMilkshapeModel::Destroy()
{
	if(m_pGroups)
	{
		for(int i=0;i<m_NumGroups;i++)
		{
			if(m_pGroups[i].Texture)
			{
 				m_pGroups[i].Texture->Release();
				m_pGroups[i].Texture=0;
			}
			if(m_pGroups[i].VertexBuffer)
			{
				m_pGroups[i].VertexBuffer->Release();
				m_pGroups[i].VertexBuffer=0;
			}
		}
		delete[] m_pGroups;
		m_pGroups=0;
	}
	m_NumGroups=0;
	//return success
	return(S_OK);
}