//CW3DControl.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include the header file
#include "../headers/CW3DControl.h"

//constructor/deconstructor
//sets defaults and releases memory
CW3DControl::CW3DControl()
{
	//set defaults
	hr=0;
	m_NumTriangles=0;
	m_pTriangles[0]=0;
	m_pTriangles[1]=0;
	m_pTriangles[2]=0;
	m_vecPos=D3DXVECTOR3(0,0,0);
	m_vecPrevPos=D3DXVECTOR3(0,0,0);
	m_Angle=0;
	m_Speed=0.0f;
}
CW3DControl::~CW3DControl()
{
	//release memory
	Destroy();
}
//loads up the bounding level for collisions with the terrain.
//then it sets up the input and gets the controls all ready
HRESULT CW3DControl::Initialize(HWND hWnd)
{
	//destroy the control if it was already created
	Destroy();

	//load up the level into a data structure
	sMilkshapeData* pData=LoadMilkshapeModel(COLLISION_LEVEL);

	//check if it loaded correctly
	if(pData==0)
		return(E_FAIL);

	//create the triangles based on the number of triangles
	m_NumTriangles=pData->pGroups[0].NumFaces;
	m_pTriangles[0]=new D3DXVECTOR3[m_NumTriangles];
	m_pTriangles[1]=new D3DXVECTOR3[m_NumTriangles];
	m_pTriangles[2]=new D3DXVECTOR3[m_NumTriangles];

	//loop and copy the triangles into their structures
	for(int i=0;i<m_NumTriangles;i++)
	{
		sFace* tri=&pData->pFaces[pData->pGroups[0].pFaceIndices[i]];
		m_pTriangles[0][i]=pData->pVertices[tri->VertexIndices[0]].vecPos;
		m_pTriangles[1][i]=pData->pVertices[tri->VertexIndices[1]].vecPos;
		m_pTriangles[2][i]=pData->pVertices[tri->VertexIndices[2]].vecPos;
	}

	//release the data now that we're done with it
	delete pData;

	//initialize direct input
	if(FAILED(hr=m_Input.Initialize(hWnd)))
		return(hr);

	//set the initial angle and position and speed
	m_Angle=INITIAL_ANGLE;
	m_vecPos=INITIAL_POSITION;
	m_Speed=INITIAL_SPEED;
	
	//loop and find the sin and cosing lookup values
	for(i=0;i<360;i++)
	{
		m_SinLookup[i]=(float)sin(DegToRad(i));
		m_CosLookup[i]=(float)cos(DegToRad(i));
	}
	
	//return success
	return(S_OK);
}
//processes the input, speed, position, and angle values
//also counts the collisions too. oh yea, it also adds
//the wind values to the position
HRESULT CW3DControl::Process()
{
	//first set the previous position to the current position
	m_vecPrevPos=m_vecPos;

	//next determine which keys are down
	BYTE keys[256];
	DIMOUSESTATE mouse;
	if(FAILED(hr=m_Input.Process((BYTE**)&keys,&mouse)))
		return(hr);
	BOOL bLeft=keys[LEFT_BUTTON]&0x80;
	BOOL bRight=keys[RIGHT_BUTTON]&0x80;
	BOOL bUp=keys[UP_BUTTON]&0x80;

	//check for the escape key
	if(keys[ESCAPE_BUTTON]&0x80)
		PostQuitMessage(0);

	//copy keys into member variables
	m_bUp=bUp;
	m_bLeft=bLeft;
	m_bRight=bRight;

	//increment the speed if holding up. if not, decrease.
	if(bUp)
		m_Speed+=SPEED_INCREMENT;
	else
		m_Speed-=SPEED_INCREMENT;

	//check if it's within the range of the min and max speeds
	if(m_Speed>MAX_SPEED)
		m_Speed=MAX_SPEED;
	if(m_Speed<MIN_SPEED)
		m_Speed=MIN_SPEED;

	//modify the angle if the left or right button was pressed
	//we also don't move as much if the boat isn't moving as fast
	//just like real life is. you can't turn if you're still
	float interpolation=(float)m_Speed/(float)MAX_SPEED;
	if(bLeft)
		m_Angle-=LEFT_INCREMENT*interpolation;
	if(bRight)
		m_Angle+=RIGHT_INCREMENT*interpolation;

	//make sure the angle is within range
	if(m_Angle<0.0f)
		m_Angle=360.0f;
	if(m_Angle>360.0f)
		m_Angle=0.0f;


	//figure the new position based on the wind and angle and speed
	m_vecPos+=D3DXVECTOR3(m_Speed*m_SinLookup[(int)m_Angle],
							RAY_CAST_HEIGHT,
						m_Speed*m_CosLookup[(int)m_Angle])+WIND;

	//now loop through the triangles and cast a ray to determine where
	//the height should be. if it doesn't find a new height value, then
	//we set the new position to the current one
	float height=RAY_CAST_HEIGHT+1000,u,v;
	for(int i=0;i<m_NumTriangles;i++)
	{
		D3DXIntersectTri(&m_pTriangles[0][i],//points of tri
							&m_pTriangles[1][i],
							&m_pTriangles[2][i],
							&m_vecPos,//pos
							&D3DXVECTOR3(0,-1,0),//dir
							&u,&v,&height);
	}
	//now check if the height is less than the water height
	//if it is, then set it to the previous position
	float dist=RAY_CAST_HEIGHT-height;
	if(dist<WATER_HEIGHT)
		m_vecPos.y=WATER_HEIGHT;
	else
		//set the old height because it's on land
		m_vecPos=m_vecPrevPos;
	

	//return success
	return(S_OK);
}
//returns values as pointers
void CW3DControl::GetValues(D3DXVECTOR3 &pPosition,float &pAngle,BOOL &Up,BOOL &Left,BOOL &Right)
{
	//set the values
	pPosition=m_vecPos;
	pAngle=m_Angle;
	Up=m_bUp;
	Left=m_bLeft;
	Right=m_bRight;
}
//releases the class's memory
HRESULT CW3DControl::Destroy()
{
	//release the input object
	m_Input.Destroy();

	//release the triangles
	if(m_pTriangles[0])
	{
		delete[] m_pTriangles[0];
		m_pTriangles[0]=0;
	}
	if(m_pTriangles[1])
	{
		delete[] m_pTriangles[1];
		m_pTriangles[1]=0;
	}
	if(m_pTriangles[2])
	{
		delete[] m_pTriangles[2];
		m_pTriangles[2]=0;
	}

	//return success
	return(S_OK);
}